#include "3048f.h"
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <limits.h>
#include <stdlib.h>

#define ENABLE  asm ("andc.b #127,ccr")
/* CLEAR INTERRUPT MASK Bit=0 enabled*/

#define DISABLE  asm ("orc.b #128,ccr")
/* SET INTERRUPT MASK bit=1 disabled */

/* this type "u32" is for 32-bit high resolution timer 
from two 16-bit timers ITU0 (lsb) and ITU1 (msb) */

union u32_t
{
  struct
  {
    unsigned msb;		/* higher 16 bits */
    unsigned lsb;		/* lower 16 bits */
  }
  sep2;
  unsigned long tog;
};
typedef union u32_t u32;

union s32_t
{
  struct
  {
    int msw;		/* higher 16 bits */
    int lsw;		/* lower 16 bits */
  }
  words;
  long s32;
};
typedef union s32_t s32;

/* this structure is for user-modifiable parameters */
struct param_t
{
  int min, max;			/* -8192 ... 8191 */
  unsigned char name[17];	/* Display name (with unit) */
  unsigned char code[4];	/* short code (serial input used) */
  float factor, offset;		/* to convert from integer to display float */
  unsigned sign, left, right;	/* digits to display of float */
  void (*updfunc) (void);	/* pointer to update function if changed */
  void (*dispfunc) (int, char *);	/* pointer to display conversion function */
};
typedef struct param_t param;
#define MODE (parval [0])
#define SSPD (parval [1])
#define SMOD (parval [2])
#define SFUN (parval [3])
#define TSMP (parval [4])
#define FAST (parval [5])
#define OFS1 (parval [6])
#define OFS2 (parval [7])
#define OFS3 (parval [8])
#define OFS4 (parval [9])
#define OFS5 (parval[10])
#define OFS6 (parval[11])
#define OFS7 (parval[12])
#define OFS8 (parval[13])
#define PLOK (parval[14])
#define PLHI (parval[15])
#define PCOK (parval[16])
#define PCHI (parval[17])
#define GN1  (parval[18])
#define GN2  (parval[19])
#define GN3  (parval[20])
#define GN4  (parval[21])
#define GN5  (parval[22])
#define GN6  (parval[23])
#define POL1 (parval[24])
#define POL2 (parval[25])
#define POL3 (parval[26])
#define POL4 (parval[27])
#define POL5 (parval[28])
#define POL6 (parval[29])
#define SRL1 (parval[30])
#define SRL2 (parval[31])
#define SRL3 (parval[32])
#define SRL4 (parval[33])
#define SRL5 (parval[34])
#define SRL6 (parval[35])
#define FC1  (parval[36])
#define FC2  (parval[37])
#define FC3  (parval[38])
#define FC4  (parval[39])
#define FC5  (parval[40])
#define FC6  (parval[41])
#define TWT  (parval[42])		     
#define TRMP (parval[43])		     
		     
/* convert.c ghh 7 July 2000*/
void utoa5 (char *s, unsigned u);
int itoa6 (char *s, int i);
void floatprint (unsigned char *s, float val, unsigned sign, unsigned left,
		 unsigned right, unsigned outlen);
void ultoa10 (char *s, unsigned long u);
void ltoa11 (char *s, long i);
int myatoi (char *s, int len, int *conv);
int myatof (char *s, int len, float *conv);
void uppercase (unsigned char *s, unsigned len);
int namecmp (const char  *s1, const char  *s2, int len1, int len2);
int namecmpbra (const char *s1, const char  *s2, int len1, int len2);
void uctoa8 (char *s, unsigned char c);

/* errno.c */
int *__errno (void);

/* datest.c ghh 22 Nov 2000 */
void statusled (unsigned char status);
void led (unsigned char status);
int main (void);
void finetime (u32 * up);
void ioinit (void);
void __attribute__ ((interrupt_handler)) int_irq0 (void);
void processinput (unsigned char *buf, unsigned len);
void showall (void);
void dacinit (void);
void dacset (unsigned char channel, unsigned val);
void adinit (void);

/* lcd1.c ghh 17. June 2000 */
void LCD_out8 (unsigned char data);
void LCD_out4_command (unsigned char data);
void LCD_out4_data (unsigned char data);
void waitloop (unsigned int n);

/* lcd.c ghh 16.May 2000 */
void LCD_dsp (unsigned const char *str, int line);
void LCD_upd (unsigned const char *str, int line);
void lcdinit (void);

/* serial.c ghh  Sat Jun 24 21:50:46 JST 2000 */
void sinit (void);
void serspeedname (int serspeed, char *s);
void serfunname (int serfun, char *s);
void sermodename (int sermode, char *s);
void sendfloat (float x);
void sendint (int x);
void sendulong (unsigned long x);
int sputs (unsigned const char *s, int l);
void sputch (unsigned char ch);
void __attribute__ ((interrupt_handler)) int_txi1 (void);
void __attribute__ ((interrupt_handler)) int_eri1 (void);
void __attribute__ ((interrupt_handler)) int_rxi1 (void);

/* store_restore.S */
int disablestore (void);
void restoreenable (int val);

/* aamc.c */
void statusled (unsigned char status);
void led (unsigned char __attribute__ ((unused)) status);
int main (void);
void finetime (u32 * up);
void adinit (void);
void ioinit (void);
void __attribute__ ((interrupt_handler)) int_irq0 (void);
void processinput (unsigned char *buf, unsigned len);
void showall (void);
void dacinit (void);
void dacset (unsigned char channel, unsigned val);
void __attribute__ ((interrupt_handler)) int_adi (void);
void fsampname (int i, char *s);
void modename (int i, char *s);
void t3init (void);
void __attribute__ ((interrupt_handler)) int_imia3 (void);
void chthrs (void);
void iirinit (void);
void newmode(void);
void fsmpinit(void);
int satadd (long *sum, long delta);
int limit (long *var, long limit);
void ofsinit(void);
void dacset6 (unsigned val[6]);

/* from disp.c */
void lcdout (void);

/* from smul.S */
long smul (long x, int y);

