/* aamc.c ghh Mon Dec  4 21:38:54 JST 2000 */
/* digital filter for TAMA modecleaner alignment */

#include "aamc.h"

#define BCDSWITCH (P4.DR.BYTE & 0x0f)
#define RESETKEY (P4.DR.BIT.B4)
#define MAINSWITCH1 (!P4.DR.BIT.B5)
#define MAINSWITCH2 (!P4.DR.BIT.B6)
#define MAINSWITCH3 (!P4.DR.BIT.B7)

#define DAC1LD (P5.DR.BIT.B2)
#define DAC1CS (P5.DR.BIT.B3)
#define DAC2LD (P5.DR.BIT.B0)
#define DAC2CS (P5.DR.BIT.B1)

#define TWOPI 6.283185307
#define LONG2PERCENT 4.656612873e-08

#define MX11 -3603
#define MX12 -4165
#define MX13 4137
#define MX21 -3603
#define MX22 2918
#define MX23 4137
#define MX31 -69
#define MX32 -437
#define MX33 2902
#define MY11 -6029
#define MY12 4160
#define MY13 -5820
#define MY21 6008
#define MY22 4160
#define MY23 -5820
#define MY31 5
#define MY32 -6734
#define MY33 2903

#define NAVG 2048

extern volatile char sersend, serrcv;

param par[] = {
  {0, 0, 5, "Mode            ", "MODE", 1., 0., 0, 2, 0, newmode, modename},
  {0, 0, 4, "Serial Speed[bd]", "SSPD", 1., 0., 0, 0, 0, sinit, serspeedname}
  ,
  {0, 0, 15, "Serial Port Mode", "SMOD", 1., 0., 0, 0, 0, sinit, sermodename}
  ,
  {0, 0, 3, "Serial Function ", "SFUN", 1., 0., 0, 0, 0, sinit, serfunname}
  ,
  {250, 100, 8191, "sampl. time [ms]", "TSMP", 4e-3, 0., 0, 2, 3, fsmpinit,
   NULL},
  {1, 0, 1, "AD Fast=1 Slow=0", "FAST", 1., 0., 0, 1, 0, adinit,
   NULL},
  {22, -2047, 2047, "Offset 1 [% FS] ", "OFS1", 100. / 2047., 0, 1, 3, 3,
   ofsinit, NULL},
  {30, -2047, 2047, "Offset 2 [% FS] ", "OFS2", 100. / 2047., 0, 1, 3, 3,
   ofsinit, NULL},
  {21, -2047, 2047, "Offset 3 [% FS] ", "OFS3", 100. / 2047., 0, 1, 3, 3,
   ofsinit, NULL},
  {34, -2047, 2047, "Offset 4 [% FS] ", "OFS4", 100. / 2047., 0, 1, 3, 3,
   ofsinit, NULL},
  {25, -2047, 2047, "Offset 5 [% FS] ", "OFS5", 100. / 2047., 0, 1, 3, 3,
   ofsinit, NULL},
  {13, -2047, 2047, "Offset 6 [% FS] ", "OFS6", 100. / 2047., 0, 1, 3, 3,
   ofsinit, NULL},
  {0, -2047, 2047, "Offset 7 [% FS] ", "OFS7", 100. / 2047., 0, 1, 3, 3,
   ofsinit, NULL},
  {0, -2047, 2047, "Offset 8 [% FS] ", "OFS8", 100. / 2047., 0, 1, 3, 3,
   ofsinit, NULL},
  {64, 0, 256, "P_L OK   [% FS] ", "PLOK", 100. / 256., 0, 1, 3, 1,
   ofsinit, NULL},
  {250, 0, 256, "P_L High [% FS] ", "PLHI", 100. / 256., 0, 1, 3, 1,
   ofsinit, NULL},
  {64, 0, 256, "P_C OK   [% FS] ", "PCOK", 100. / 256., 0, 1, 3, 1,
   ofsinit, NULL},
  {250, 0, 256, "P_C High [% FS] ", "PCHI", 100. / 256., 0, 1, 3, 1,
   ofsinit, NULL}
};
#define MODE (par[0].val)
#define SSPD (par[1].val)
#define SMOD (par[2].val)
#define SFUN (par[3].val)
#define TSMP (par[4].val)
#define FAST (par[5].val)
#define OFS1 (par[6].val)
#define OFS2 (par[7].val)
#define OFS3 (par[8].val)
#define OFS4 (par[9].val)
#define OFS5 (par[10].val)
#define OFS6 (par[11].val)
#define OFS7 (par[12].val)
#define OFS8 (par[13].val)
#define PLOK (par[14].val)
#define PLHI (par[15].val)
#define PCOK (par[16].val)
#define PCHI (par[17].val)

static const unsigned nparam = sizeof (par) / sizeof (param);
/* number of parameters */

static unsigned iparam = 0;
/* index of next parameter to be changed by rotary switch */

static const char hello1[16] = "MC alignment    ";
  /*                            0123456789012345  */
static const char hello2[16] = "v0.02 2000.01.31";

#include "br.c"			/* bit reverse table */

unsigned volatile adsum[8];
unsigned volatile char adn0, adn1, adgroup, adfast;

int volatile addata[8];
unsigned volatile char newdata = 0;	/* 1 if new A/D data is ready */
unsigned volatile char toofast = 0;
	/* 1 if new A/D data arrives when old has
	   not yet been used (A/D data arrives too fast for user) */

#define TIM3CLOCK 2e6		/* Frequency of timer 3 (phi/8) */
#define TWOTHIRD 1431655765	/* 2/3 of full scale for long int */

void
led (unsigned char __attribute__ ((unused)) status)
{
/*  P1.DR.BYTE = ~status; */
}

static volatile long accu[8];	/* loop output */
static volatile long fact[8];	/* precomputed gain  factor */
static volatile long srlim[8];	/* precomputed slew-rate limit */
static volatile long insum[8];	/* input sum for offset average */
static volatile int offset[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };	/* input offset */

static long nin;
static int insuminvalid;

static unsigned char m4ch, m4lch, m4mode;

inline int
mydiv (long a, int b)
{
  long result;
asm ("divxs.w %1,%0": "=r" (result): "r" (b), "0" (a):"cc");
  return (int) result;
}

inline long
mymul (int a, int b)
{
  long result;
asm ("mulxs.w %1,%0": "=r" (result): "r" (b), "0" (a):"cc");
  return result;
}



int
main (void)
{
  unsigned char str1[17], str2[17];
  int i;
  u32 time;
  unsigned char ledno, outbyte;
  unsigned outword;
  int err[6], in[6];
  unsigned char led1, led2;
  char plok, pcok, plhi, pchi;
  long ltemp;
  int itemp, c1, c2;

  memset (str1, ' ', 16);
  memset (str2, ' ', 16);
  led (81);
  lcdinit ();
  led (82);
  ioinit ();
  ENABLE;
  led (83);
  sinit ();
  led (84);
  dacinit ();
  adinit ();
  ofsinit ();
  accu[0] = 0;
  accu[1] = 0;
  accu[2] = 0;
  accu[3] = 0;
  led (85);
  t3init ();
  ENABLE;
  led (86);
  /* Welcome message */
  led (87);
  LCD_dsp (hello1, 0);
  led (88);
  LCD_dsp (hello2, 1);
  led (89);

  led (1);
  for (;;)
    {
      switch (MODE)
	{
	case 0:		/* Normal */
	  while (!newdata)
	    ;			/* get new data */

	  led1 = led2 = 0;
	  /* check power levels */
	  if ((plok = ((unsigned) addata[6] / 16 >= (unsigned) PLOK)))
	    led1 |= 0x40;
	  if ((pcok = ((unsigned) addata[7] / 16 >= (unsigned) PCOK)))
	    led1 |= 0x80;
	  if ((plhi = ((unsigned) addata[6] / 16 >= (unsigned) PLHI)))
	    led2 |= 0x40;
	  if ((pchi = ((unsigned) addata[7] / 16 >= (unsigned) PCHI)))
	    led2 |= 0x80;
	  P1.DR.BYTE = ~led1;
	  P2.DR.BYTE = ~led2;

	  P6.DR.BYTE = 0x01;	/* status 1 */
	  ltemp = (long) addata[6] * (long) addata[7];
/*	  c1 = (int) (sqrt ((float) ltemp)); */
	  c1= c2 = addata[7];
	  P6.DR.BYTE = 0x02;	/* status 1 */

	  itemp = addata[0] - offset[0];
	  in[0] = mydiv (mymul (3996, itemp), c1);
	  itemp = addata[1] - offset[1];
	  in[1] = mydiv (mymul (3996, itemp), c1);
	  itemp = addata[2] - offset[2];
	  in[2] = mydiv (mymul (3996, itemp), c1);
	  itemp = addata[3] - offset[3];
	  in[3] = mydiv (mymul (3996, itemp), c1);
	  itemp = addata[4] - offset[4];
	  in[4] = mydiv (mymul (3996, itemp), c2);
	  itemp = addata[5] - offset[5];
	  in[5] = mydiv (mymul (3996, itemp), c2);
	  P6.DR.BYTE = 0x04;	/* status 2 */
	  ltemp =
	    (long) (MX11 * (long) in[0]) +
	    (long) (MX12 * (long) in[2]) + (long) (MX13 * (long) in[4]);
	  err[0] = mydiv (ltemp, 4096);
	  ltemp =
	    (long) (MX21 * (long) in[0]) +
	    (long) (MX22 * (long) in[2]) + (long) (MX23 * (long) in[4]);
	  err[1] = mydiv (ltemp, 4096);
	  ltemp =
	    (long) (MX31 * (long) in[0]) +
	    (long) (MX32 * (long) in[2]) + (long) (MX33 * (long) in[4]);
	  err[2] = mydiv (ltemp, 4096);
	  ltemp =
	    (long) (MY11 * (long) in[1]) +
	    (long) (MY12 * (long) in[3]) + (long) (MY13 * (long) in[5]);
	  err[3] = mydiv (ltemp, 4096);
	  ltemp =
	    (long) (MY21 * (long) in[1]) +
	    (long) (MY22 * (long) in[3]) + (long) (MY23 * (long) in[5]);
	  err[4] = mydiv (ltemp, 4096);
	  ltemp =
	    (long) (MY31 * (long) in[1]) +
	    (long) (MY32 * (long) in[3]) + (long) (MY33 * (long) in[5]);
	  err[5] = mydiv (ltemp, 4096);
	  P6.DR.BYTE = 0x08;	/* status 0 */
	  err[0] = err[0] + 2046;
	  if (err[0] < 0)
	    err[0] = 0;
	  err[1] = err[1] + 2046;
	  if (err[1] < 0)
	    err[1] = 0;
	  err[2] = err[2] + 2046;
	  if (err[2] < 0)
	    err[2] = 0;
	  err[3] = err[3] + 2046;
	  if (err[3] < 0)
	    err[3] = 0;
	  err[4] = err[4] + 2046;
	  if (err[4] < 0)
	    err[4] = 0;
	  err[5] = err[5] + 2046;
	  if (err[5] < 0)
	    err[5] = 0;
	  dacset6 ((unsigned *) err);
	  P6.DR.BYTE = 0x00;
	  if (toofast)
	    P1.DR.BYTE = 0x00;
	  newdata = 0;
	  break;
	case 1:		/* LED Test */
	  led (3);
	  memset (str1, ' ', 16);
	  memcpy (str1, "Input:", 6);
	  outbyte = P4.DR.BYTE;	/* read input ports */
	  uctoa8 (str1 + 7, outbyte);	/* convert to "00100" binary */
	  LCD_dsp (str1, 0);
	  finetime (&time);
	  time.tog /= 1000000UL;	/* 0.5 seconds */
	  ledno = (unsigned) (time.tog % 16);
	  memset (str2, ' ', 16);
	  memcpy (str2, "LED", 3);
	  utoa5 (str2 + 4, (unsigned) ledno);
	  LCD_dsp (str2, 1);
	  if (ledno < 8)
	    {
	      outbyte = (0x01 << ledno);
	      P1.DR.BYTE = ~outbyte;
	      P2.DR.BYTE = ~0x00;
	    }
	  else
	    {
	      outbyte = (0x01 << (ledno - 8));
	      P1.DR.BYTE = ~0x00;
	      P2.DR.BYTE = ~outbyte;
	    }
	  break;
	case 2:		/* DAC Test */
	  led (4);
	  /* Generate 1 Hz ramp */
	  for (i = 0; i < 8; i++)
	    {
	      finetime (&time);
	      outword = (time.tog / 488 + i * 512) % 4096;
	      dacset (i, outword);
	    }
	  break;
	case 3:		/* Unity Gain Test */
	  led (90);
	  while (!newdata);	/* get new data */
	  led (91);
	  dacset (0, (addata[0] - offset[0]) + 2046);
	  dacset (1, (addata[1] - offset[1]) + 2046);
	  dacset (2, (addata[2] - offset[2]) + 2046);
	  dacset (3, (addata[3] - offset[3]) + 2046);
	  dacset (4, (addata[4] - offset[4]) + 2046);
	  dacset (5, (addata[5] - offset[5]) + 2046);
	  dacset (6, (addata[6] - offset[6]) + 2046);
	  dacset (7, (addata[7] - offset[7]) + 2046);
	  led (92);
	  if (toofast)
	    P1.DR.BYTE = 0x00;
	  newdata = 0;
	  break;
	case 4:		/* Average Input */
	  switch (m4mode)
	    {
	    case 0:
	    case 127:		/* waiting for reset key (start signal) */
	      memcpy (str1, "Average Input   ", 16);
	      memcpy (str2, "'Reset' to start", 16);
	      LCD_upd (str1, 0);
	      LCD_upd (str2, 1);
	      if (RESETKEY)
		{

		  insum[0] = 0;
		  insum[1] = 0;
		  insum[2] = 0;
		  insum[3] = 0;
		  insum[4] = 0;
		  insum[5] = 0;
		  insum[6] = 0;
		  insum[7] = 0;
		  nin = 0;
		  insuminvalid = 0;
		  m4mode = 1;
		}
	      break;
	    case 1:		/* averaging */
	      while (!newdata);	/* wait for new AD data */
	      ++nin;
	      insum[0] += addata[0] - offset[0];
	      insum[1] += addata[1] - offset[1];
	      insum[2] += addata[2] - offset[2];
	      insum[3] += addata[3] - offset[3];
	      insum[4] += addata[4] - offset[4];
	      insum[5] += addata[5] - offset[5];
	      insum[6] += addata[6] - offset[6];
	      insum[7] += addata[7] - offset[7];
	      if (nin == NAVG)
		{
		  led1 = 0xC0;	/* set both green LED */
		  m4mode = 2;
		  break;
		}
	      else if (nin > 64)
		led1 = 0x40;	/* set first green LED */
	      memcpy (str2, "Running...      ", 16);
	      LCD_upd (str2, 1);
	      newdata = 0;
	      if (toofast)
		P1.DR.BYTE = P2.DR.BYTE = 0x00;
	      else
		P1.DR.BYTE = P2.DR.BYTE = 0xFF;
	      break;
	    case 2:
	      memcpy (str1, "Done.      Press", 16);
	      LCD_upd (str1, 0);
	      memcpy (str2, "'Reset' to  save", 16);
	      LCD_upd (str2, 1);
	      if (sersend)
		{
		  memcpy (str1, "OFS1=      ", 11);
		  itoa6 (str1 + 5, (offset[0] + insum[0] / nin));
		  while (sputs (str1, 11));
		  sputch ('\n');
		  memcpy (str1, "OFS2=      ", 11);
		  itoa6 (str1 + 5, (offset[1] + insum[1] / nin));
		  while (sputs (str1, 11));
		  sputch ('\n');
		  memcpy (str1, "OFS3=      ", 11);
		  itoa6 (str1 + 5, (offset[2] + insum[2] / nin));
		  while (sputs (str1, 11));
		  sputch ('\n');
		  memcpy (str1, "OFS4=      ", 11);
		  itoa6 (str1 + 5, (offset[3] + insum[3] / nin));
		  while (sputs (str1, 11));
		  sputch ('\n');
		  memcpy (str1, "OFS5=      ", 11);
		  itoa6 (str1 + 5, (offset[4] + insum[4] / nin));
		  while (sputs (str1, 11));
		  sputch ('\n');
		  memcpy (str1, "OFS6=      ", 11);
		  itoa6 (str1 + 5, (offset[5] + insum[5] / nin));
		  while (sputs (str1, 11));
		  sputch ('\n');
		  memcpy (str1, "OFS7=      ", 11);
		  itoa6 (str1 + 5, (offset[6] + insum[6] / nin));
		  while (sputs (str1, 11));
		  sputch ('\n');
		  memcpy (str1, "OFS8=      ", 11);
		  itoa6 (str1 + 5, (offset[7] + insum[7] / nin));
		  while (sputs (str1, 11));
		  sputch ('\n');
		}
	      m4mode = 3;
	      break;
	    case 3:
	      if (RESETKEY)
		{
		  m4mode = 4;
		}
	      break;
	    case 4:
	      offset[0] += (insum[0] / nin);
	      offset[1] += (insum[1] / nin);
	      offset[2] += (insum[2] / nin);
	      offset[3] += (insum[3] / nin);
	      offset[4] += (insum[4] / nin);
	      offset[5] += (insum[5] / nin);
	      offset[6] += (insum[6] / nin);
	      offset[7] += (insum[7] / nin);
	      OFS1 = offset[0];
	      OFS2 = offset[1];
	      OFS3 = offset[2];
	      OFS4 = offset[3];
	      OFS5 = offset[4];
	      OFS6 = offset[5];
	      OFS7 = offset[6];
	      OFS8 = offset[7];
	      if (sersend)
		{
		  memcpy (str1, "Offsets saved.", 14);
		  while (sputs (str1, 14));
		  sputch ('\n');
		}
	      m4mode = 0;
	      while (RESETKEY);
	      /* wait 0.5 seconds after key release */
	      for (i = 0; i < 50; i++)
		waitloop (10000);
	      break;
	    }
	  break;
	case 5:		/* read power */
	  while (!newdata);	/* get new data */
	  led1 = led2 = 0;
	  /* check power levels */
	  if ((plok = ((unsigned) addata[6] / 256 >= (unsigned) PLOK)))
	    led1 |= 0x40;
	  if ((pcok = ((unsigned) addata[7] / 256 >= (unsigned) PCOK)))
	    led1 |= 0x80;
	  if ((plhi = ((unsigned) addata[6] / 256 >= (unsigned) PLHI)))
	    led2 |= 0x40;
	  if ((pchi = ((unsigned) addata[7] / 256 >= (unsigned) PCHI)))
	    led2 |= 0x80;
	  P1.DR.BYTE = ~led1;
	  P2.DR.BYTE = ~led2;
	  floatprint (str1,
		      (float) addata[6] * (100. / (4. * 1023.)), 1, 3, 3, 16);
	  floatprint (str2,
		      (float) addata[7] * (100. / (4. * 1023.)), 1, 3, 3, 16);
	  LCD_upd (str1, 0);
	  LCD_upd (str2, 1);
	  break;
	}
    }
  return 0;			/* silence compiler */
}



void
finetime (u32 * up)
/* reads time in units of 0.5 usec */
{
  int irqstatus = disablestore ();
  ITU.TSTR.BYTE &= ~0x03;	/* stop timers 0/1 */
  up->sep2.msb = ITU1.TCNT;	/* read timer counter */
  up->sep2.lsb = ITU0.TCNT;
  ITU.TSTR.BYTE |= 0x03;	/* restart timers 0/1 */
  restoreenable (irqstatus);
}

void
adinit (void)
{
  int i;
  led (32);
  /* AD converter */
  for (i = 0; i < 8; i++)
    adsum[i] = 0;
  adn0 = adn1 = 0;
  led (33);
  AD.CR.BYTE = 0x7f;		/* no ext. trigger */
  AD.CSR.BYTE = 0x0;		/* clear all funcions & stop converter */
  led (34);
  adgroup = 0;
  if ((adfast = FAST))
    AD.CSR.BYTE = 0x7b;		/* interrupt, start, scan, fast, ch 0-3 */
  else
    AD.CSR.BYTE = 0x73;		/* interrupt, start, scan, slow, ch 0-3 */
  led (35);
}

void
ioinit (void)
{
  P1.DDR = 0xff;		/* port 1 output (LED) */
  P1.DR.BYTE = 0xff;		/* clear LED */
  P2.DDR = 0xff;		/* port 2 output (LED) */
  P2.DR.BYTE = 0xff;		/* clear LED */
/*  P3 is LCD port; see lcd.c */
  P4.DDR = 0x00;		/* port 4 input (comparators etc) */
  P4.PCR.BYTE = 0xFF;		/* port 4 input pull-up active */
  P5.DR.BYTE = 0xff;		/* DAC signals all high */
  P5.DDR = 0xff;		/* port 5 output (DAC) */
  P6.DDR = 0xff;		/* port 6 output (Debugging) */
  P6.DR.BYTE = 0x00;		/* start with all zeroes */
  DA.STCR.BYTE = 0xff;		/* standby mode keep D/A running */
  DA.CR.BYTE = 0x1f;		/* DA0 and DA1 off */
  /* up/down counter in timer channel 2 */
  ITU.TMDR.BYTE = 0x40;		/* phase counting mode */
  ITU.TSNC.BYTE = 0x00;		/* timers not synchronized */
  ITU.TFCR.BYTE = 0x00;		/* normal mode channels 3 and 4 */
/* enable interrupt 0 */
  INTC.IER.BYTE = 0x01;		/* enable */
  INTC.ISCR.BYTE = 0x01;	/* falling edge */
  P8.DDR = 0x0;			/* port 8 input mode */
  ITU0.TCNT = 0;		/* start timers at 0 */
  ITU1.TCNT = 0;
  ITU0.TCR.BYTE = 0x03;		/* timer 0 clock phi/8 */
  ITU1.TCR.BYTE = 0x16;		/* timer 1 clock TCLKC, count both edges */
  ITU0.TIOR.BYTE = 0x03;	/* timer 0 toggle at GRA compare match */
  ITU0.GRA = 0xffff;		/* timer 0 GRA */
  ITU.TSTR.BYTE |= 0x03;	/* start timers 0 and 1 */
}


/* IRQ0 handler (key pressed) */
void __attribute__ ((interrupt_handler)) int_irq0 (void)
{
  unsigned char msg[17];
  unsigned char dummy;
  int val4, lastval, lastval4;
  param *pp;
  u32 time, lasttime;
  unsigned long difftime;
  float logmaxspeed, logratio, logspeed;
  int maxstep, step;
  unsigned char ibuf[6];
  int i;
  int oldval;
  dummy = INTC.ISR.BYTE;
  dummy &= ~0x01;		/* clear IRQ */
  INTC.ISR.BYTE = dummy;
  /* verify parameter number */
  if (iparam > nparam)
    iparam = nparam;
  if (iparam == nparam)		/* Display welcome message */
    {
      LCD_dsp (hello1, 0);
      LCD_dsp (hello2, 1);
      /* loop while key remains pressed */
      while ((P8.DR.BYTE & 0x01) == 0);
      iparam = 0;
      return;
    }

  /* pointer to parameter structure */
  pp = &(par[iparam]);
  /* precompute constants for speed calculation */
  logmaxspeed = 5.3;		/* log 200 */
  maxstep = (pp->max - pp->min) / 8;
  if (maxstep < 1)
    maxstep = 1;
  logratio = log (maxstep) / logmaxspeed;
  /* Write name of parameter to display */
  LCD_dsp (pp->name, 0);
  /* Write starting value of parameter to display */
  if (pp->dispfunc == NULL)
    floatprint (msg, pp->val * pp->factor + pp->offset,
		pp->sign, pp->left, pp->right, 16);
  else
    {
      memset (msg, ' ', 16);
      (pp->dispfunc) (pp->val, msg);
    }
  memcpy (msg + 12, pp->code, 4);
  LCD_dsp (msg, 1);
  /* preset loop variables */
  lastval = oldval = pp->val;
  /* The counter ITU2 counts 4 steps for one 
     true step of the rotary switch, hence the counter
     value in ITU2.TCNT4 is maintained at 4 times 
     the value of the integer parameter pp->val */
  lastval4 = val4 = 4 * lastval;
  ITU2.TCNT = val4;		/* preset counter */
  ITU.TSTR.BYTE |= 0x04;	/* start channel 2 counting */
  finetime (&lasttime);
  /* loop while key remains pressed */
  while ((P8.DR.BYTE & 0x01) == 0)
    {
      val4 = ((int) ITU2.TCNT);	/* read counter value */
      if (val4 < lastval4 - 3 || val4 > lastval4 + 3)
	/* 4 small steps = 1 true step */
	{
	  /* measure time difference to laststep */
	  finetime (&time);
	  difftime = (time.tog - lasttime.tog) / 4096;
	  lasttime = time;
	  /* compute step via turning speed */
	  logspeed = logmaxspeed - log (difftime);
	  step = (int) (exp (logspeed * logratio) + .5);
	  if (step < 1)
	    step = 1;
	  if (step > maxstep)
	    step = maxstep;
	  /* update parameter */
	  {
	    /* temporarily compute difference to avoid comparison errors
	       near upper limit of 'int' range */

	    int valdiff = val4 - lastval4;
	    if (valdiff < 0)	/* step down ? */
	      step = -step;
	  }
	  pp->val = lastval + step;
	  /* clip parameter */
	  if (pp->val < pp->min)
	    pp->val = pp->min;
	  if (pp->val > pp->max)
	    pp->val = pp->max;
	  /* write clipped counter value */
	  if (pp->val != lastval)
	    {

	      if (pp->dispfunc == NULL)
		floatprint (msg, pp->val * pp->factor + pp->offset,
			    pp->sign, pp->left, pp->right, 16);
	      else
		{
		  memset (msg, ' ', 16);
		  (pp->dispfunc) (pp->val, msg);
		}

	      memcpy (msg + 12, pp->code, 4);
	      LCD_upd (msg, 1);
	    }
	  lastval = pp->val;
	  /*    rewrite to counter */
	  ITU2.TCNT = lastval4 = 4 * pp->val;
	}
    }
  ITU.TSTR.BYTE &= ~0x04;	/* stop channel 2 counting */
  if (sersend)
    {
      /* write new parameter to serial port */
      while (sputs (&((pp->name)[0]), 16));
      sputch ('=');
      while (sputs (msg, 16));
      sputch ('=');
      i = itoa6 (ibuf, pp->val);
      while (sputs (ibuf, i));
      if (oldval != pp->val)
	{
	  memcpy (msg, "(Old=", 5);
	  i = itoa6 (msg + 5, oldval);
	  msg[5 + i] = ')';
	  while (sputs (msg, 6 + i));
	}
      sputch ('\n');
    }
  if (pp->val != oldval)	/* val was changed */
    {
      if (pp->updfunc != NULL)	/* initialization function for this parameter */
	pp->updfunc ();
    }
  else
    ++iparam;			/* next keypress goes to next parameter */
}


void
processinput (unsigned char *buf, unsigned len)
{
  unsigned char msg[16], ibuf[6];
  int conv, oldval;
  float fconv;
  unsigned int i, j = 0, found = 0;
  if (!serrcv)
    return;
  /* check 'show' special command */
  if (namecmp ("SHOW", buf, 4, len) == 0)
    /* command 'show' (print all parameters) */
    {
      showall ();
      return;
    }


  /* find '=' */
  for (j = 0; j < len && buf[j] != '='; j++);
  if (buf[j] != '=')
    goto ser_err;
  /* first check 4-letter codes */
  for (i = 0; i < nparam; i++)
    if (namecmp (par[i].code, buf, sizeof (par[i].code), j) == 0)
      {
	found = 1;
	break;
      }
  if (!found)			/* check long parameter names */
    {

      for (i = 0; i < nparam; i++)
	{
	  if (namecmpbra (par[i].name, buf, sizeof (par[i].name), j) == 0)
	    {
	      found = 1;
	      break;
	    }
	}
    }

  if (!found)			/* still nothing found */
    goto ser_err;
  for (; buf[j] == ' ' || buf[j] == '\t'; j++);	/* skip whitespace */
  if (buf[j] != '=')
    goto ser_err;
  else
    ++j;
  if (myatof (buf + j, len - j, &fconv))
    goto ser_err;
  if (fconv < par[i].min * par[i].factor + par[i].offset)
    conv = par[i].min;
  else if (fconv > par[i].max * par[i].factor + par[i].offset)
    conv = par[i].max;
  else
    conv = (fconv - par[i].offset) / par[i].factor;
  if (conv < par[i].min)
    conv = par[i].min;
  if (conv > par[i].max)
    conv = par[i].max;
  oldval = par[i].val;
  if (oldval != conv)		/* change */
    {
      par[i].val = conv;
      if (par[i].updfunc != NULL)
	par[i].updfunc ();
    }
  LCD_dsp (par[i].name, 0);
  if (par[i].dispfunc == NULL)
    floatprint (msg,
		par[i].val * par[i].factor +
		par[i].offset, par[i].sign, par[i].left, par[i].right, 16);
  else
    {
      memset (msg, ' ', 16);
      (par[i].dispfunc) (par[i].val, msg);
    }

  memcpy (msg + 12, par[i].code, 4);
  LCD_dsp (msg, 1);
  if (sersend)
    {
      /* write new parameter to serial port */
      while (sputs (&((par[i].name)[0]), 16));
      sputch ('=');
      while (sputs (msg, 16));
      sputch ('=');
      i = itoa6 (ibuf, par[i].val);
      while (sputs (ibuf, i));
      if (oldval != par[i].val)
	{
	  memcpy (msg, "(Old=", 5);
	  i = itoa6 (msg + 5, oldval);
	  msg[5 + i] = ')';
	  while (sputs (msg, 6 + i));
	}
      sputch ('\n');
    }
  return;
ser_err:
  memcpy (msg, "Serial Input Err", 16);
  LCD_dsp (msg, 0);
  if (sersend)
    {
      while (sputs (msg, 16));
      sputch (':');
    }
  memset (msg, ' ', 16);
  if (len > 16)
    len = 16;
  memcpy (msg, buf, len);
  LCD_dsp (msg, 1);
  if (sersend)
    {
      while (sputs (buf, len));
      sputch ('\n');
    }
}

void
showall (void)
{
  int i, len = 40;
  char msg[17], ibuf[7];
  if (!sersend)
    return;
  sputch ('#');
  while (sputs (hello1, 16));
  sputch ('\n');
  sputch ('#');
  while (sputs (hello2, 16));
  sputch ('\n');
  for (i = 0; i < (int) nparam; i++)
    {
      while (sputs (&((par[i].name)[0]), 16));
      sputch ('=');
      if (par[i].dispfunc == NULL)
	floatprint (msg,
		    par[i].val * par[i].factor +
		    par[i].offset, par[i].sign, par[i].left,
		    par[i].right, 16);
      else
	{
	  memset (msg, ' ', 16);
	  (par[i].dispfunc) (par[i].val, msg);
	}
      memcpy (msg + 12, par[i].code, 4);
      while (sputs (msg, 16));
      sputch ('=');
      len = itoa6 (ibuf, par[i].val);
      while (sputs (ibuf, len));
      sputch ('\n');
    }
}

void
dacinit (void)
{
  P5.DR.BYTE |= 0xff;		/* set all bits high */
/* Initialize serial port */
  SCI0.SCR.BYTE = 0x0;
  /* disable everything for setting */
  SCI0.SMR.BYTE = 0x80;
  /* synchronous, clock phi */
  SCI0.BRR = 0x1;
  /* clock rate = 2Mbit/sec (!) */
  waitloop (1);			/* wait 2 bits (1 required) */
  SCI0.SCR.BYTE = 0x20;
  /* enable transmit , internal clock, clock output, no interrupts */
  dacset (0, 0x800);
  dacset (1, 0x800);
  dacset (2, 0x800);
  dacset (3, 0x800);
  dacset (4, 0x800);
  dacset (5, 0x800);
  dacset (6, 0x800);
  dacset (7, 0x800);
}

void
dacset (unsigned char channel, unsigned val)
{
  unsigned send16;
  unsigned char send8;
  unsigned char dac = channel / 4;
  if (val > 0x0fff)
    val = 0x0fff;		/* clip */
/* put together 16 bits (12+2+2*0) */
  send16 = (val & 0x0fff) | ((channel & 0x3) << 14);
  if (dac == 0)
    DAC1CS = 0;			/* set CS\ low */
  else
    DAC2CS = 0;
/* AD8420 needs highest bit first,
this CPU sends lowest first, reverse ! */
  send8 = br[send16 / 256];
/* wait until serial port not busy */
  while (!SCI0.SSR.BIT.TDRE);	/* wait as long as not empty */
  SCI0.TDR = send8;
/* send higher byte, bits reversed */
  SCI0.SSR.BIT.TDRE = 0;
/* indicate arrival of data to be sent */
/* prepare next byte */
  send8 = br[send16 & 0xff];
/* wait until serial port not busy */
  while (!SCI0.SSR.BIT.TDRE);	/* wait as long as not empty */
  SCI0.TDR = send8;
/* send higher byte, bits reversed */
  SCI0.SSR.BIT.TDRE = 0;
/* indicate arrival of data to be sent */
/* wait until last bit is sent */
  while (!SCI0.SSR.BIT.TEND);	/* wait until sent */
  if (dac == 0)
    {
      DAC1CS = 1;		/* set CS\ high */
      DAC1LD = 0;		/* set LD\ low */
      DAC1LD = 1;		/* set LD\ high */
    }

  else
    {
      DAC2CS = 1;		/* set CS\ high */
      DAC2LD = 0;		/* set LD\ low */
      DAC2LD = 1;		/* set LD\ high */
    }
}



void
t3init (void)
{
  led (16);
  ITU.TSTR.BIT.STR3 = 0;	/* stop timer 3 */
  ITU3.TCNT = 0;		/* reset */
  ITU3.GRA = 8U * (unsigned) TSMP - 1;	/* set frequency */
  ITU3.TCR.BYTE = 0x23;		/* Phi/8=2MHz, clear on GRA */
  ITU3.TIOR.BYTE = 0x03;	/* GRA output compare; toggle */
  ITU3.TIER.BYTE = 0x01;	/* GRA compare match interrupt enable */
  ITU.TSTR.BIT.STR3 = 1;	/* start timer 3 */
  led (17);
}



void
fsmpinit (void)
{
  t3init ();
}

int
satadd (long *sum, long delta)
/* addition with saturation  returns 1 if overflow, 0 if ok */
{
  if (*sum > 0 && delta > 0 && *sum + delta < 0)
    {
      *sum = LONG_MAX;
      return 1;
    }
  else if (*sum < 0 && delta < 0 && *sum + delta > 0)
    {
      *sum = LONG_MIN;
      return 1;
    }
  else
    *sum += delta;
  return 0;
}

int
limit (long *var, long limit)
/* 	limit *var to the range -limit <= *var <= *limit,
returns 1 if cropping was active, 0 otherwise */
{
  if (*var > limit)
    {
      *var = limit;
      return 1;
    }
  else if (*var < -limit)
    {
      *var = -limit;
      return 1;
    }
  return 0;
}

void
modename (int i, char *s)
{
  led (12);
  memset (s, ' ', 16);
  switch (i)
    {
    case 0:
      memcpy (s, "Normal", 6);
      break;
    case 1:
      memcpy (s, "LED-Test", 8);
      break;
    case 2:
      memcpy (s, "DAC-Test", 8);
      break;
    case 3:
      memcpy (s, "Gain=1", 6);
      break;
    case 4:
      memcpy (s, "Offs Adj", 8);
      break;
    case 5:
      memcpy (s, "Read Power", 10);
      break;
    default:
      memcpy (s, "Illegal:", 8);
      itoa6 (s + 8, i);
      break;
    }
}

void
newmode (void)
{
  char str[16];
  led (13);
  memcpy (str, "newmode:       \n", 16);
  itoa6 (str + 8, MODE);
  while (sputs (str, 16));
  switch (MODE)
    {
    case 0:			/* Normal */
      LCD_dsp (hello1, 0);
      LCD_dsp (hello2, 1);
      break;
    case 1:			/* LED Test */
      break;
    case 2:			/* DAC Test */
      LCD_dsp (hello1, 0);
      memcpy (str, "DAC Test        ", 16);
      LCD_dsp (str, 1);
      break;
    case 3:			/* Unity Gain Test */
      LCD_dsp (hello1, 0);
      memcpy (str, "Unity Gain Test ", 16);
      LCD_dsp (str, 1);
      break;
    case 4:			/* Average Input */
      m4ch = m4mode = m4lch = 127;	/* illegal/unused */
      break;
    case 5:
      break;
    }
}

void
ofsinit (void)
{
  offset[0] = OFS1;
  offset[1] = OFS2;
  offset[2] = OFS3;
  offset[3] = OFS4;
  offset[4] = OFS5;
  offset[5] = OFS6;
  offset[6] = OFS7;
  offset[7] = OFS8;
}

#define WAITDAC while (!SCI0.SSR.BIT.TDRE);
#define WAITDACEND while (!SCI0.SSR.BIT.TEND);

void
dacset6 (unsigned val[6])
{
  unsigned send16;
  unsigned char send8;
  DAC1CS = 0;			/* set CS\ low */
  if (val[0] > 0x0fff)
    val[0] = 0x0fff;		/* clip */
  send16 = (val[0] & 0x0fff);
  send8 = br[send16 / 256];	/* byte 1 ready */
  WAITDAC;
  SCI0.TDR = send8;
  SCI0.SSR.BIT.TDRE = 0;	/* byte 1 sending */
  send8 = br[send16 & 0xff];	/* byte 2 ready */
  WAITDAC;
  SCI0.TDR = send8;
  SCI0.SSR.BIT.TDRE = 0;	/* byte 2 sending */
  if (val[1] > 0x0fff)
    val[1] = 0x0fff;		/* clip */
  send16 = (val[1] & 0x0fff) | 0x4000;
  send8 = br[send16 / 256];	/* byte 3 ready */
  WAITDACEND;
  DAC1LD = 0;			/* set LD\ low */
  DAC1LD = 1;			/* set LD\ high */
  SCI0.TDR = send8;
  SCI0.SSR.BIT.TDRE = 0;	/* byte 3 sending */
  send8 = br[send16 & 0xff];	/* byte 4 ready */
  WAITDAC;
  SCI0.TDR = send8;
  SCI0.SSR.BIT.TDRE = 0;	/* byte 4 sending */
  if (val[2] > 0x0fff)
    val[2] = 0x0fff;		/* clip */
  send16 = (val[2] & 0x0fff) | 0x8000;
  send8 = br[send16 / 256];	/* byte 5 ready */
  WAITDACEND;
  DAC1LD = 0;			/* set LD\ low */
  DAC1LD = 1;			/* set LD\ high */
  SCI0.TDR = send8;
  SCI0.SSR.BIT.TDRE = 0;	/* byte 5 sending */
  send8 = br[send16 & 0xff];	/* byte 6 ready */
  WAITDAC;
  SCI0.TDR = send8;
  SCI0.SSR.BIT.TDRE = 0;	/* byte 6 sending */
  if (val[3] > 0x0fff)
    val[3] = 0x0fff;		/* clip */
  send16 = (val[3] & 0x0fff) | 0xC000;
  send8 = br[send16 / 256];	/* byte 7 ready */
  WAITDACEND;
  DAC1LD = 0;			/* set LD\ low */
  DAC1LD = 1;			/* set LD\ high */
  SCI0.TDR = send8;
  SCI0.SSR.BIT.TDRE = 0;	/* byte 7 sending */
  send8 = br[send16 & 0xff];	/* byte 8 ready */
  WAITDAC;
  SCI0.TDR = send8;
  SCI0.SSR.BIT.TDRE = 0;	/* byte 8 sending */
  if (val[4] > 0x0fff)
    val[4] = 0x0fff;		/* clip */
  send16 = (val[4] & 0x0fff);
  send8 = br[send16 / 256];
  WAITDACEND;
  DAC1CS = 1;			/* set CS\ high */
  DAC1LD = 0;			/* set LD\ low */
  DAC1LD = 1;			/* set LD\ high */
  DAC2CS = 0;			/* set CS\ low */
  SCI0.TDR = send8;
  SCI0.SSR.BIT.TDRE = 0;
  send8 = br[send16 & 0xff];
  WAITDAC;
  SCI0.TDR = send8;
  SCI0.SSR.BIT.TDRE = 0;
  if (val[5] > 0x0fff)
    val[5] = 0x0fff;		/* clip */
  send16 = (val[5] & 0x0fff) | 0x4000;
  send8 = br[send16 / 256];
  WAITDACEND;
  DAC2LD = 0;			/* set LD\ low */
  DAC2LD = 1;			/* set LD\ high */
  SCI0.TDR = send8;
  SCI0.SSR.BIT.TDRE = 0;
  send8 = br[send16 & 0xff];
  WAITDAC;
  SCI0.TDR = send8;
  SCI0.SSR.BIT.TDRE = 0;
  WAITDACEND;
  DAC2CS = 1;			/* set CS\ high */
  DAC2LD = 0;			/* set LD\ low */
  DAC2LD = 1;			/* set LD\ high */
}
