/* serial.c ghh  Sun Dec 10 21:50:47 JST 2000 */
/* serial output functions */

#include "aamc.h"

extern volatile int parval[];

#define BUFLEN 1850
#define RBUFLEN 64
static unsigned volatile char ringbuf[BUFLEN];
static unsigned volatile bufstart = 0, buflen = 0;
static unsigned volatile char readbuf[RBUFLEN];
static unsigned volatile rbuflen = 0;

volatile char sersend = 0, serrcv = 0;

void
sinit (void)
{
  led (64);
  SCI1.SCR.BYTE = 0x0;
  led (65);
  /* disable everything for setting */
  if (SMOD < 0 || SMOD > 15)
    SMOD = 0;
  led (66);
  SCI1.SMR.BYTE = ((unsigned char) SMOD) << 3;
  led (67);
  /* asynchronous, 8-bit, no parity, (even parity),
     one stop bit, multi disabled, clock phi/1, no clock output */
  /* Bit 6 Databits 0=8    1=7  
     Bit 5 Parity   0=No   1=Yes
     Bit 4 Parity   0=Even 1=Odd     
     Bit 3 Stop     0=One  1=Two */

/*   Number 0...15 << 3
     Numb  B6 B5 B4 B3 Name
       0    0  0  0  0 8N1
       1    0  0  0  1 8N2
       2    0  0  1  0 8N1
       3    0  0  1  1 8N2
       4    0  1  0  0 8E1
       5    0  1  0  1 8E2
       6    0  1  1  0 8O1
       7    0  1  1  1 8O2
       8    1  0  0  0 7N1
       9    1  0  0  1 7N2
      10    1  0  1  0 7N1
      11    1  0  1  1 7N2
      12    1  1  0  0 7E1
      13    1  1  0  1 7E2
      14    1  1  1  0 7O1
      15    1  1  1  1 7O2
*/

  switch (SSPD)
    {
    case 0:
    default:
      SCI1.BRR = 12;
      /* 38400 bit/sec */
      break;

    case 1:
      SCI1.BRR = 25;
      /* 19200 bit/sec */
      break;

    case 2:
      SCI1.BRR = 51;
      /* 9600 bit/sec */
      break;

    case 3:
      SCI1.BRR = 103;
      /* 4800 bit/sec */
      break;

    case 4:
      SCI1.BRR = 207;
      /* 2400 bit/sec */
      break;
    }
  led (68);
  /* baud SCI1.BRR  (16 MHz clock) */
  /*  2400 207 */
  /*  4800 103 */
  /*  9600 51 */
  /* 19200 25 */
  /* 38400 12 */

  if (SFUN == 0)		/* Both */
    {
      led (69);
      SCI1.SCR.BYTE = 0xF0;
      sersend = serrcv = 1;
    }
  /* enable transmit and receive,
     and transmit/receive interrupts, NOT transmit-end-interrupt */
  else if (SFUN == 1)		/* Send */
    {
      led (70);
      SCI1.SCR.BYTE = 0xA0;
      rbuflen = 0;
      sersend = 1;
      serrcv = 0;
    }
  /* enable transmit,
     and transmit interrupt, NOT transmit-end-interrupt */
  else if (SFUN == 2)		/* Rcv */
    {
      led (71);
      SCI1.SCR.BYTE = 0x50;
      buflen = 0;
      sersend = 0;
      serrcv = 1;
    }
  /* enable receive, and receive interrupt */
  else				/* None */
    {
      led (72);
      SCI1.SCR.BYTE = 0x00;	/* disable everything */
      buflen = rbuflen = 0;
      sersend = serrcv = 0;
    }
}

void
serspeedname (int serspeed, char *s)
{
  static const char ssn[6][6] =
    { "38400", "19200", " 9600", " 4800", " 2400", "ERROR" };
  const char *src;
  if (serspeed < 0)
    serspeed = 5;
  if (serspeed > 4)
    serspeed = 5;
  src = &(ssn[serspeed][0]);
  *s++ = *src++;
  *s++ = *src++;
  *s++ = *src++;
  *s++ = *src++;
  *s = *src;
}

void
serfunname (int serfun, char *s)
{
  static const char ssn[5][5] = {
    "Both",			/* 0 */
    "Send",			/* 1 */
    "Recv",			/* 2 */
    "None",			/* 3 */
    "ERR "
  };

  /* Send if 0 || 1
     Receive if 0 || 2 */

  const char *src;
  if (serfun < 0)
    serfun = 4;
  if (serfun > 3)
    serfun = 4;
  src = &(ssn[serfun][0]);
  *s++ = *src++;
  *s++ = *src++;
  *s++ = *src++;
  *s = *src;
}

void
sermodename (int sermode, char *s)
{
  static const char ssn[17][4] = { "8N1",
    "8N2",
    "8N1",
    "8N2",
    "8E1",
    "8E2",
    "8O1",
    "8O2",
    "7N1",
    "7N2",
    "7N1",
    "7N2",
    "7E1",
    "7E2",
    "7O1",
    "7O2",
    "ERR"
  };

  const char *src;
  if (sermode < 0)
    sermode = 16;
  if (sermode > 15)
    sermode = 16;
  src = &(ssn[sermode][0]);
  *s++ = *src++;
  *s++ = *src++;
  *s = *src;
}

void
sendfloat (float x)
{
  union
  {
    float f;
    unsigned char c[4];
  }
  u;

  u.f = x;
  while (sputs (&(u.c[0]), 4));
}

void
sendint (int x)
{
  union
  {
    int f;
    unsigned char c[2];
  }
  u;

  u.f = x;
  while (sputs (&(u.c[0]), 2));
}

void
sendulong (unsigned long x)
{
  union
  {
    unsigned long f;
    unsigned char c[4];
  }
  u;

  u.f = x;
  while (sputs (&(u.c[0]), 4));
}

int
sputs (unsigned const char *s, int l)
{
  int i = l;

  if (!sersend)			/* send disabled */
    return 0;
  if (l + buflen > BUFLEN)
    return 1;			/* buffer overflow */
/* disable transmit interrupt */
  SCI1.SCR.BYTE &= ~0x80;
/* add new string to ringbuffer */
  for (; i > 0; i--)
    {
/*      if (*s == '\n')
	*s = 0x0d;	*/	/* translate CR for #$`#`$`@@@ WIndows */
      ringbuf[(bufstart + buflen++) % BUFLEN] = *(s++);

    }
/* re-enable transmit interrupt */
  SCI1.SCR.BYTE |= 0x80;
  return 0;
}

void
sputch (unsigned char ch)
{
  if (!sersend)
    return;
  if (buflen >= BUFLEN)		/* buffer overflow */
    {
      /* re-enable transmit interrupt */
      SCI1.SCR.BYTE |= 0x80;
      /* wait until 1 char can fit into buffer */
      while (buflen >= BUFLEN);
    }
/* disable transmit interrupt */
  SCI1.SCR.BYTE &= ~0x80;
/* add new char to ringbuffer */
  ringbuf[(bufstart + buflen++) % BUFLEN] = ch;
/* re-enable transmit interrupt */
  SCI1.SCR.BYTE |= 0x80;
}

/* serial transmit interrupt handler */
void __attribute__ ((interrupt_handler)) int_txi1 (void)
{
  led (127);
  if (!sersend)
    {
      led (128);
      buflen = 0;
      SCI1.SSR.BYTE &= ~0x80;	/* clear interrupt */
      return;
    }
  led (129);
  if (buflen > 0)
    {
      SCI1.TDR = ringbuf[bufstart];
      bufstart = (bufstart + 1) % BUFLEN;
      --buflen;
      /* write character */
      SCI1.SSR.BYTE &= ~0x80;
      /* clear `empty` bit - start transmission */
    }
  led (130);
  if (buflen == 0)
    {
      /* disable transmit interrupt */

      SCI1.SCR.BYTE &= ~0x80;
      SCI1.SSR.BYTE &= ~0x80;	/* clear interrupt */
    }
  led (131);
}

void __attribute__ ((interrupt_handler)) int_eri1 (void)
{
  led (132);
  SCI1.SSR.BYTE &= ~0x38;	/* clear error bits */
}

void __attribute__ ((interrupt_handler)) int_rxi1 (void)
{
  unsigned char c, lcdm[16];
  led (133);
  SCI1.SSR.BYTE &= ~0x40;	/* clear full bit */
  led (134);
  if (!serrcv)
    return;
  led (135);
  c = SCI1.RDR;
  if (rbuflen >= RBUFLEN)
    {
      led (136);
      /* overflow */
      memcpy (lcdm, "Ser.Inp.Overfl.:", 16);
      sputs (lcdm, 16);
      LCD_dsp (lcdm, 0);
      memset (lcdm, ' ', 16);
      lcdm[0] = c;
      utoa5 (lcdm + 2, (unsigned) c);
      LCD_dsp (lcdm, 1);
      rbuflen = 0;
      return;
    }
  led (137);
  if (c == 0x0a || c == 0x0d || c == '\n')	/* newline */
    {
      led (138);
      if (rbuflen > 0)
	processinput ((unsigned char *) readbuf, rbuflen);
      /* typecast to remove 'volatile'; interrupt disabled here */
      rbuflen = 0;
      return;
    }
  led (139);
  readbuf[rbuflen++] = c;
  return;
}
